/*
 * vfdclock - a simple clock program which outputs to a
 *            Matrix Orbital VFD2041 vacuum fluorescent display.
 *
 * This program is in the public domain -- do whatever you want with it.
 *
 * David Simmons
 * http://davidsimmons.com/
 * July 1, 2010
 */

#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <string.h>

#include "vfd.h"

#define BAUDRATE B19200
#define DEVICENAME "/dev/ttyUSB0"
#define DATABITS CS8
#define STOPBITS 0
#define PARITY 0
#define PARITYON 0

struct vfd_struct {
    int fd;
};

int vfd_init(vfd_t *vfd_pointer) {
    vfd_t vfd = calloc(1, sizeof(struct vfd_struct));
    if (vfd == NULL) {
        perror("calloc");
        exit(1);
    }

    vfd->fd = open(DEVICENAME, O_RDWR | O_NOCTTY);
    if (vfd->fd == -1) {
        perror("opening serial port");
        exit(1);
    }

    struct termios oldtio, newtio;
    tcgetattr(vfd->fd,&oldtio);
    newtio.c_cflag = BAUDRATE | CRTSCTS | DATABITS | STOPBITS | PARITYON | PARITY | CLOCAL | CREAD;
    newtio.c_iflag = IGNPAR;
    newtio.c_oflag = 0;
    newtio.c_lflag = 0;
    newtio.c_cc[VMIN]=1;
    newtio.c_cc[VTIME]=0;
    tcflush(vfd->fd, TCIFLUSH);
    tcsetattr(vfd->fd,TCSANOW,&newtio);

    *vfd_pointer = vfd;

    vfd_brightness(vfd, 3);
    vfd_cursor_blink(vfd, 0);

    return 0;
}

int vfd_close(vfd_t vfd) {
    close(vfd->fd);
    free(vfd);
    return 0;
}

int vfd_write_zero(vfd_t vfd) {
    write(vfd->fd, "\0", 1);
    return 0;
}

int vfd_write(vfd_t vfd, const char *message) {
    write(vfd->fd, message, strlen(message));
    return 0;
}

int vfd_clear(vfd_t vfd) {
    write(vfd->fd, "\xFE\x58", 2);
    return 0;
}

/* 0 = most bright, 3 = most dim */
int vfd_brightness(vfd_t vfd, int level) {
    unsigned char data[3] = {0xFE, 0x59};
    data[2] = level & 0x03;
    write(vfd->fd, data, sizeof(data));
    return 0;
}

int vfd_cursor_position(vfd_t vfd, int row, int column) {
    unsigned char data[4] = {0xFE, 0x47};
    data[2] = column;
    data[3] = row;
    write(vfd->fd, data, sizeof(data));
    return 0;
}

int vfd_cursor_home(vfd_t vfd) {
    unsigned char data[2] = {0xFE, 0x48};
    write(vfd->fd, data, sizeof(data));
    return 0;
}

int vfd_cursor_blink(vfd_t vfd, int enable) {
    unsigned char data[2] = {0xFE};
    if (enable) {
        data[1] = 0x53;
    } else {
        data[1] = 0x54;
    }
    write(vfd->fd, data, sizeof(data));
    return 0;
}

int vfd_cursor_left(vfd_t vfd) {
    unsigned char data[2] = {0xFE, 0x4C};
    write(vfd->fd, data, sizeof(data));
    return 0;
}

int vfd_cursor_right(vfd_t vfd) {
    unsigned char data[2] = {0xFE, 0x4D};
    write(vfd->fd, data, sizeof(data));
    return 0;
}

int vfd_clock(vfd_t vfd, int hours, int minutes) {

    unsigned char data[] = {
        0xFE, 0x6E,
        0xFE, 0x47, 0x0A, 0x02, 0x04,
        0xFE, 0x47, 0x0A, 0x03, 0x04,
        0xFE, 0x23, 0x06, 0x00,
        0xFE, 0x23, 0x0C, 0x00,
        0xFE, 0x23, 0x10, 0x00
    };

    unsigned char draw_first_digit[] = {
        0xFE, 0x23, 0x02, 0x00
    };

    unsigned char erase_first_digit[] = {
        0xFE, 0x47, 0x02, 0x01, 0x20, 0x20, 0x20,
        0xFE, 0x47, 0x02, 0x02, 0x20, 0x20, 0x20,
        0xFE, 0x47, 0x02, 0x03, 0x20, 0x20, 0x20,
        0xFE, 0x47, 0x02, 0x04, 0x20, 0x20, 0x20,
    };

    if (hours < 10) {
        write(vfd->fd, erase_first_digit, sizeof(erase_first_digit));
    } else {
        draw_first_digit[3] = hours / 10;
        write(vfd->fd, draw_first_digit, sizeof(draw_first_digit));
    }
    data[15] = hours % 10;
    data[19] = minutes / 10;
    data[23] = minutes % 10;

    write(vfd->fd, data, sizeof(data));
}

